// ignore_for_file: avoid_classes_with_only_static_members

import 'dart:convert';
import 'dart:math';

import 'package:http/http.dart';

import 'package:sghi_core/communities/core/endpoints.dart';
import 'package:sghi_core/communities/core/utils.dart';
import 'package:sghi_core/communities/models/user.dart';
import 'package:sghi_core/flutter_graphql_client/constants.dart';
import 'package:sghi_core/flutter_graphql_client/i_flutter_graphql_client.dart';

class ChatAPI {
  /// Creates a Matrix room
  static Future<Response> createRoom({
    required String roomName,
    required String topic,
    required IGraphQlClient client,
  }) async {
    final String alias = roomName.replaceAll(' ', '');

    final Map<String, dynamic> variables = <String, dynamic>{
      'visibility': 'private',
      'is_direct': false,
      'preset': 'private_chat',
      'invite': <String>[
        // Add a list of users to invite to this group
      ],
      'name': roomName,
      'room_alias_name': alias,
      'topic': topic,
    };

    final Response response = await client.callRESTAPI(
      endpoint: createRoomEndpoint,
      method: kPostMethodName,
      variables: variables,
    );

    return response;
  }

  /// Deletes a room message
  static Future<Response> deleteMessage({
    required String roomID,
    required String eventID,
    required String senderID,
    required String userID,
    required IGraphQlClient client,
  }) async {
    final String tempTxnID = Random.secure().nextInt(1 << 32).toString();

    final Response response = await client.callRESTAPI(
      endpoint: deleteMessageEventEndpoint(roomID, tempTxnID, eventID),
      method: kPutMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Returns a processed matrix image URL
  ///
  /// Basically converts a MXC URI to an actual image URL
  static String? getMediaURL({required String? mediaUrl}) {
    if (mediaUrl?.isEmpty ?? true) {
      return null;
    }

    final String? mediaID = mediaUrl?.split('/').last;

    if (mediaID?.isNotEmpty ?? false) {
      return fetchMediaEndpoint(mediaID!);
    }

    return null;
  }

  /// Returns a list of all members of a room
  static Future<List<RoomUser>> fetchRoomMembers({
    required String roomID,
    required IGraphQlClient client,
  }) async {
    final String url = joinedMembersEndpoint(roomID);

    final Map<String, dynamic> powerLevels =
        await fetchRoomPowerLevels(roomID, client);

    final Map<String, dynamic>? powerLevelUser =
        powerLevels['users'] as Map<String, dynamic>?;

    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    final Map<String, dynamic> decodedResponse =
        json.decode(response.body) as Map<String, dynamic>;

    final Map<String, dynamic> members =
        decodedResponse['joined'] as Map<String, dynamic>;

    final Map<dynamic, RoomUser> users =
        Map<dynamic, dynamic>.from(members).map(
      (dynamic key, dynamic value) {
        final Map<String, dynamic> v = value as Map<String, dynamic>;

        int? userLevel = 0;

        if (powerLevelUser?.containsKey(key) ?? false) {
          userLevel = powerLevelUser?[key.toString()] as int;
        }

        return MapEntry<dynamic, RoomUser>(
          key,
          RoomUser(
            powerLevel: userLevel,
            moderator: userLevel >= 50,
            roomUser: User(
              userID: key.toString(),
              avatarUri: v['avatar_url'] as String?,
              displayName: v['display_name'] as String?,
            ),
          ),
        );
      },
    );

    return users.values.toList();
  }

  /// Invites a user into a room
  static Future<Response> inviteUsers({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: inviteUserToRoomEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{'user_id': userID},
    );

    return response;
  }

  /// Joins a Matrix room
  static Future<Response> joinRoom({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: joinRoomEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Removes a user from a room
  static Future<Response> kickUser({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
    String? reason,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: kickUserEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{
        'reason': reason ?? 'Removed by a moderator',
        'user_id': userID
      },
    );

    return response;
  }

  /// Reports a message.
  ///
  /// This marks a message as inappropriate and gives the message the lowest score
  /// of -100
  static Future<Response> reportMessage({
    required IGraphQlClient client,
    required String roomID,
    required String eventID,
    String? reason,
  }) async {
    final String url = reportMessageEndpoint(roomID: roomID, eventID: eventID);

    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kPostMethodName,
      variables: <String, dynamic>{
        'reason': reason ?? 'Inappropriate content',
        'score': -100,
      },
    );

    return response;
  }

  /// Lists all messages that have been reported by users in the room
  static Future<Response> fetchReportedEvents({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: reportedEventsEndpoint(roomID),
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Fetches more details of a message report
  static Future<Response> fetchMessageReport({
    required IGraphQlClient client,
    required String reportID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: eventReportByIDEndpoint(reportID),
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Deletes a report for a message
  static Future<Response> deleteMessageReport({
    required IGraphQlClient client,
    required String reportID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: eventReportByIDEndpoint(reportID),
      method: kDeleteMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Leaves a room
  ///
  /// A user will not be able to access the room once they have left it
  static Future<Response> leaveRoom({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: leaveRoomEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Searches for members to add into a room
  static Future<Response> searchGroupMembers({
    required IGraphQlClient client,
    required String searchParam,
    int limit = 10,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: searchMembersEndpoint,
      method: kPostMethodName,
      variables: <String, dynamic>{'limit': limit, 'search_term': searchParam},
    );
    return response;
  }

  /// Signs in to a Matrix home server
  static Future<Response> signIn({
    required IGraphQlClient client,
    required String? username,
    required String? password,
  }) async {
    const String loginAuthType = 'm.login.password';

    final Map<String, dynamic> variables = <String, dynamic>{
      'type': loginAuthType,
      'identifier': <String, dynamic>{'type': 'm.id.user', 'user': username},
      'password': password,
    };

    final Response response = await client.callRESTAPI(
      endpoint: loginEndpoint,
      method: kPostMethodName,
      variables: variables,
    );

    return response;
  }

  /// Sends a message to a room
  static Future<Response> sendMessage({
    required IGraphQlClient client,
    required String roomID,
    required String message,
  }) async {
    final String tempTxnID = Random.secure().nextInt(1 << 32).toString();

    final Response response = await client.callRESTAPI(
      endpoint: sendMessageEventEndpoint(roomID, tempTxnID),
      method: kPutMethodName,
      variables: <String, dynamic>{'body': message, 'msgtype': 'm.text'},
    );

    return response;
  }

  /// Sends a media message
  ///
  /// Media in this case is only restricted to Images
  static Future<Response> sendMediaMessage({
    required IGraphQlClient client,
    required String roomID,
    required String messageText,
    required String imageMXCUrl,
  }) async {
    final String tempTxnID = Random.secure().nextInt(1 << 32).toString();

    final Response response = await client.callRESTAPI(
      endpoint: sendMessageEventEndpoint(roomID, tempTxnID),
      method: kPutMethodName,
      variables: <String, dynamic>{
        'body': messageText,
        'msgtype': 'm.image',
        'url': imageMXCUrl,
        'info': <String, dynamic>{
          'mimetype': 'image/jpeg',
          'size': 12345,
          'thumbnail_info': <String, dynamic>{
            'mimetype': 'image/jpeg',
            'size': 6789,
          },
          'thumbnail_url': imageMXCUrl,
          'w': 640,
          'h': 480,
          'external_info': <String, dynamic>{
            'caption': messageText,
          },
        },
      },
    );

    return response;
  }

  /// Promotes a user to a moderator
  ///
  /// It basically assigns the power level of a user to 50
  static Future<Response> promoteToModerator({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: promoteUserToModeratorEndpoint(roomID),
      method: kPutMethodName,
      variables: <String, dynamic>{
        'users': <String, dynamic>{userID: 50},
      },
    );

    return response;
  }

  /// Bans a user from a room. The user is also removed from this room and
  /// will not be able to join until they are unbanned.
  static Future<Response> banUser({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
    String? reason,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: banUserEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{
        'reason': reason ?? 'Banned by a moderator',
        'user_id': userID
      },
    );

    return response;
  }

  /// Lists all banned members for a specific room
  static Future<Response> fetchBannedMembers({
    required String roomID,
    required IGraphQlClient client,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: bannedMembersEndpoint(roomID),
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Removes a ban placed on a user from a room
  ///
  /// The user will still need to be invited in order to join the room
  static Future<Response> unBanUser({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: unBanUserEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{'user_id': userID},
    );
    return response;
  }
}
