import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:http/http.dart';
import 'package:http/http.dart' as http;
import 'package:image_picker/image_picker.dart';
import 'package:mockito/mockito.dart';
import 'package:sghi_core/communities/core/endpoints.dart';
import 'package:sghi_core/flutter_graphql_client/flutter_graphql_client.dart';
import 'package:sghi_core/flutter_graphql_client/i_flutter_graphql_client.dart';

import 'response_mocks.dart';

final Map<String, dynamic> loginResponseMock = <String, dynamic>{
  'user_id': '@abiudrn:chat.savannahghi.org',
  'device_id': 'some-device-id',
  'display_name': 'Abiud Orina',
  'access_token': 'some-sample-token',
  'avatar_url': 'some-avatar-url',
  'isSignedIn': true,
};

final Map<String, dynamic> messageEventMock = <String, dynamic>{
  'type': 'm.room.message',
  'sender': '@abiudrn:chat.savannahghi.org',
  'content': <String, dynamic>{
    'org.matrix.msc1767.text': 'ala!',
    'body': 'ala!',
    'msgtype': 'm.text'
  },
  'event_id': '166896610153WDxHa:chat.savannahghi.org',
  'origin_server_ts': 1668966101714,
  'unsigned': <String, dynamic>{'age': 5484013963}
};

final Map<String, dynamic> bannedUserResponseMock = <String, dynamic>{
  'chunk': <Map<String, dynamic>>[
    <String, dynamic>{
      'type': 'm.room.member',
      'room_id': '!MSXOGsoJlORHUvsaln:prohealth360.org',
      'sender': '@prohealth360:prohealth360.org',
      'content': <String, dynamic>{
        'reason': 'Inappropriate content',
        'membership': 'ban'
      },
      'state_key': '@abiudrn:prohealth360.org',
      'origin_server_ts': 1680552333276,
      'event_id': 'test-event-id',
      'user_id': '@prohealth360:prohealth360.org',
      'age': 10064,
    }
  ]
};

final Map<String, dynamic> syncResponseMock = <String, dynamic>{
  'next_batch': 'sample-next-batch',
  'presence': <String, dynamic>{
    'events': <Map<String, dynamic>>[
      <String, dynamic>{
        'type': 'm.presence',
        'sender': '@abiudrn:chat.savannahghi.org',
        'content': <String, dynamic>{
          'presence': 'online',
          'last_active_ago': 56,
          'currently_active': true
        }
      }
    ],
  },
  'rooms': <String, dynamic>{
    'join': <String, dynamic>{
      '!testRoom:chat.savannahghi.org': <String, dynamic>{
        'timeline': <String, dynamic>{
          'events': <Map<String, dynamic>>[
            <String, dynamic>{
              'type': 'm.room.message',
              'sender': '@test:chat.savannahghi.org',
              'content': <String, dynamic>{
                'org.matrix.msc1767.text': 'ala!',
                'body': 'ala!',
                'msgtype': 'm.text'
              },
              'event_id': 'test:chat.savannahghi.org',
              'origin_server_ts': 1668966101714,
            },
          ],
        },
        'state': <String, dynamic>{
          'events': <Map<String, dynamic>>[
            <String, dynamic>{
              'type': 'm.room.name',
              'sender': '@test:chat.savannahghi.org',
              'content': <String, dynamic>{'name': 'The Grand Gaming Squad'},
              'state_key': '',
              'event_id': 'test-event-id:chat.savannahghi.org',
              'origin_server_ts': 1668712693930,
              'unsigned': <String, dynamic>{'age': 5737421747}
            },
          ],
        },
        'unread_notifications': <String, dynamic>{
          'notification_count': 0,
          'highlight_count': 0
        },
        'summary': <String, dynamic>{
          'm.joined_member_count': 3,
          'm.invited_member_count': 0
        },
      }
    },
    'invite': <String, dynamic>{
      '!NvYSqaASzlfRpFCMtr:chat.savannahghi.org': <String, dynamic>{
        'invite_state': <String, dynamic>{
          'events': <Map<String, dynamic>>[
            <String, dynamic>{
              'type': 'm.room.topic',
              'state_key': '',
              'content': <String, dynamic>{
                'topic':
                    "A group about Arsenal's thrashing of Manchester United (3-2)"
              },
              'sender': '@test:chat.savannahghi.org'
            },
          ],
        }
      }
    },
    'knock': <String, dynamic>{},
    'leave': <String, dynamic>{},
  }
};

/// a short client for providing custom responses
///
/// a good use case is when you want to return error responses
class MockShortGraphQlClient extends IGraphQlClient {
  MockShortGraphQlClient();
  MockShortGraphQlClient.withResponse(
    String idToken,
    String endpoint,
    this.response,
  ) {
    super.idToken = idToken;
    super.endpoint = endpoint;
  }

  late final Response response;

  @override
  Future<Response> callRESTAPI({
    required String endpoint,
    required String method,
    Map<String, dynamic>? variables,
    Duration? timeout,
    Map<String, String>? customHeaders,
  }) {
    return Future<Response>.value(response);
  }

  @override
  Future<Response> query(String queryString, Map<String, dynamic> variables,
      {Duration? timeout}) {
    return Future<Response>.value(response);
  }
}

class MockGraphQlClient extends Mock implements GraphQlClient {
  @override
  Future<http.Response> callRESTAPI({
    required String endpoint,
    required String method,
    Map<String, dynamic>? variables,
    Map<String, String>? customHeaders,
    Duration? timeout,
  }) async {
    if (endpoint.contains(loginEndpoint)) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(loginResponseMock),
          201,
        ),
      );
    }

    if (endpoint.contains(syncEndpoint)) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(syncResponseMock),
          201,
        ),
      );
    }

    if (endpoint.contains('joined_members')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(joinedMembersResponseMock),
          201,
        ),
      );
    }

    if (endpoint.contains('/send/m.room.message/')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'event_id': '16741180035XJnPW:chat.savannahghi.org',
          }),
          201,
        ),
      );
    }

    if (endpoint.contains('m.room.power_levels')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(powerLevelsResponseMock),
          201,
        ),
      );
    }

    if (endpoint.contains('/invite')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{}),
          201,
        ),
      );
    }

    if (endpoint.contains('/join')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'room_id': '!NvYSqaASzlfRpFCMtr:chat.savannahghi.org',
          }),
          201,
        ),
      );
    }

    if (endpoint.contains('/leave')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{}),
          201,
        ),
      );
    }

    if (endpoint.contains('/kick')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{}),
          201,
        ),
      );
    }

    if (endpoint.contains('/redact')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'event_id': '16741180035XJnPW:chat.savannahghi.org',
          }),
          201,
        ),
      );
    }

    if (endpoint.contains('/createRoom')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'room_id': '!NvYSqaASzlfRpFCMtr:chat.savannahghi.org',
          }),
          201,
        ),
      );
    }

    if (endpoint.contains('/ban')) {
      return Future<http.Response>.value(
        http.Response(json.encode(<String, dynamic>{}), 201),
      );
    }

    if (endpoint.contains('/unban')) {
      return Future<http.Response>.value(
        http.Response(json.encode(<String, dynamic>{}), 201),
      );
    }

    if (endpoint.contains('/report/')) {
      return Future<http.Response>.value(
        http.Response(json.encode(<String, dynamic>{}), 201),
      );
    }

    if (endpoint.contains('/media/v3/download/')) {
      final String dir = Directory.current.path;
      final String imgPath = '$dir/test/test_resources/test_file.png';
      final ByteData imgData = await rootBundle.load(imgPath);
      final Uint8List uint8list = imgData.buffer.asUint8List();

      return Future<http.Response>.value(
        http.Response(json.encode(uint8list), 201),
      );
    }

    if (endpoint.contains(searchMembersEndpoint)) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'results': <Map<String, dynamic>>[loginResponseMock],
          }),
          201,
        ),
      );
    }

    return Future<http.Response>.value(
      http.Response(
        json.encode(<String, dynamic>{'data': <String, dynamic>{}}),
        200,
      ),
    );
  }

  @override
  Map<String, dynamic> toMap(http.Response? response) {
    final dynamic res = json.decode(response!.body);

    if (res is Map) return res as Map<String, dynamic>;
    // ignore: avoid_dynamic_calls
    return res[0] as Map<String, dynamic>;
  }
}

class ImagePickerMock extends Mock implements ImagePicker {
  @override
  Future<PickedFile?> getImage(
      {required ImageSource source,
      double? maxWidth,
      double? maxHeight,
      int? imageQuality,
      CameraDevice preferredCameraDevice = CameraDevice.rear}) {
    final String dir = Directory.current.path;
    final String imgPath = '$dir/test/test_resources/test_file.png';

    final PickedFile selectedImage = PickedFile(imgPath);

    return Future<PickedFile>.value(selectedImage);
  }
}
