import 'dart:convert';

import 'package:flutter_test/flutter_test.dart';
import 'package:http/http.dart';
import 'package:sghi_core/communities/core/chat_api.dart';
import 'package:sghi_core/communities/core/endpoints.dart';
import 'package:sghi_core/communities/models/user.dart';

import '../../mocks.dart';

void main() {
  group('ChatAPI', () {
    test('create room works correctly', () async {
      final Response response = await ChatAPI.createRoom(
        roomName: 'roomName',
        topic: 'topic',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );

      expect(response, isA<Response>());
      expect(response.statusCode, 200);

      expect(
          (jsonDecode(response.body) as Map<String, dynamic>)
              .containsKey('data'),
          true);
      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('delete message  works correctly', () async {
      final Response response = await ChatAPI.deleteMessage(
        roomID: 'roomID',
        eventID: 'eventID',
        senderID: 'senderID',
        userID: 'userID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('invite users  works correctly', () async {
      final Response response = await ChatAPI.inviteUsers(
        roomID: 'roomID',
        userID: 'userID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );

      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('join room works correctly', () async {
      final Response response = await ChatAPI.joinRoom(
        roomID: 'roomID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );

      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('kickUser works correctly', () async {
      final Response response = await ChatAPI.kickUser(
        roomID: 'roomID',
        userID: 'userID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('leaveRoom works correctly', () async {
      final Response response = await ChatAPI.leaveRoom(
        roomID: 'roomID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('searchGroupMembers works correctly', () async {
      final Response response = await ChatAPI.searchGroupMembers(
        searchParam: 'searchParam',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });
    test('fetchRoomMembers works correctly', () async {
      final List<RoomUser> users = await ChatAPI.fetchRoomMembers(
          roomID: 'roomID', client: MockGraphQlClient());
      expect(users.length, 1);
    });

    test('signIn works correctly', () async {
      final Response response = await ChatAPI.signIn(
        client: MockGraphQlClient(),
        username: 'test',
        password: 'test',
      );

      expect(json.decode(response.body)['display_name'], 'Abiud Orina');
    });

    test('sendMessage works correctly', () async {
      final Response response = await ChatAPI.sendMessage(
        roomID: 'roomID',
        message: 'testMessage',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );

      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('should send a media message correctly', () async {
      final Response response = await ChatAPI.sendMediaMessage(
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'event_id': 'event_id'}),
            200,
          ),
        ),
        imageMXCUrl: 'mxc:some-fake-mxc-url',
        messageText: 'Some fake message',
        roomID: 'a fake room ID',
      );

      expect(response.statusCode, 200);
      expect(
          (jsonDecode(response.body) as Map<String, dynamic>)
              .containsKey('event_id'),
          true);
      expect(json.decode(response.body)['event_id'], 'event_id');
    });

    test('should promote a user to a moderator', () async {
      final Response response = await ChatAPI.promoteToModerator(
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'event_id': 'event_id'}),
            200,
          ),
        ),
        roomID: 'test-room-id',
        userID: 'test-user-id',
      );

      expect(json.decode(response.body)['event_id'], 'event_id');
    });

    test('should ban a user successfully', () async {
      final Response response = await ChatAPI.banUser(
        client: MockGraphQlClient(),
        roomID: 'test-room-id',
        userID: 'test-user-id',
        reason: 'Banned',
      );

      expect(response.statusCode, 201);
    });

    test('should unban a user successfully', () async {
      final Response response = await ChatAPI.unBanUser(
        client: MockGraphQlClient(),
        roomID: 'test-room-id',
        userID: 'test-user-id',
      );

      expect(response.statusCode, 201);
    });

    test('should report a message successfully', () async {
      final Response response = await ChatAPI.reportMessage(
        client: MockGraphQlClient(),
        roomID: 'test-room-id',
        eventID: 'test-event-id',
        reason: 'Inappropriate',
      );

      expect(response.statusCode, 201);
    });

    test('should fetch banned membership events', () async {
      final Response response = await ChatAPI.fetchBannedMembers(
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(jsonEncode(bannedUserResponseMock), 200),
        ),
        roomID: 'test-room-id',
      );

      expect(response.statusCode, 200);
      expect(jsonDecode(response.body), isA<Map<String, dynamic>>());
      expect(
          (jsonDecode(response.body) as Map<String, dynamic>)
              .containsKey('chunk'),
          true);
    });

    test('should fetch reported events', () async {
      final Response response = await ChatAPI.fetchReportedEvents(
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(jsonEncode(<String, dynamic>{}), 200),
        ),
        roomID: 'test-room-id',
      );

      expect(response.statusCode, 200);
      expect(jsonDecode(response.body), isA<Map<String, dynamic>>());
    });

    test('should fetch a message report', () async {
      final Response response = await ChatAPI.fetchMessageReport(
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(jsonEncode(<String, dynamic>{}), 200),
        ),
        reportID: 'test-report-id',
      );

      expect(response.statusCode, 200);
      expect(jsonDecode(response.body), isA<Map<String, dynamic>>());
    });

    test('should delete a message report', () async {
      final Response response = await ChatAPI.deleteMessageReport(
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(jsonEncode(<String, dynamic>{}), 200),
        ),
        reportID: 'test-report-id',
      );

      expect(response.statusCode, 200);
      expect(jsonDecode(response.body), isA<Map<String, dynamic>>());
    });

    test('should get the correct media URL', () async {
      expect(ChatAPI.getMediaURL(mediaUrl: ''), isNull);
      expect(
        // ignore: prefer_single_quotes
        ChatAPI.getMediaURL(
          mediaUrl: 'mxc://chat.savannahghi.org/some-fake-id',
        ),
        fetchMediaEndpoint('some-fake-id'),
      );
    });
  });
}
